try:
    from ase.data.pubchem import pubchem_atoms_search
    from ase.io import read

    use_ase = True
except ImportError:
    use_ase = False

try:
    from rdkit.Chem import AllChem, MolFromSmiles, AddHs
    from rdkit.Chem.rdmolfiles import MolToXYZBlock

    use_rdkit = True
except ImportError:
    use_rdkit = False


def handle_io(input_string: str):
    if not use_ase and not use_rdkit:
        try:
            with open(input_string, "r") as handle:
                return read_xyz(handle)
        except (ValueError, FileNotFoundError):
            print("ERROR: Could not read '%s' as .xyz file." % input_string)
            if input_string[-4:] != ".xyz":
                print("Consider installing the optional dependencies (ase, rdkit) for more formats!")
            return False, None, None
    elif use_ase and not use_rdkit:
        try:
            atms = read(input_string)
        except FileNotFoundError:
            try:
                print("Searching '%s' on PubChem..." % input_string)
                atms = pubchem_atoms_search(smiles=input_string)
            except:
                print("ERROR: Argument does not seem to be an existing file or SMILES string.")
                return False, None, None
        return True, atms.get_positions(), atms.get_chemical_symbols()
    elif use_rdkit and not use_ase:
        try:
            with open(input_string, "r") as handle:
                return read_xyz(handle)
        except ValueError:
            print("ERROR: Could not read '%s' as xyz file." % input_string)
            return False
        except FileNotFoundError:
            proceed, pos, sym = read_smiles(input_string)
            if not proceed:
                print("ERROR: File '%s' not found and not a valid SMILES code!" % input_string)
                return False, None, None
            return proceed, pos, sym
    else:  # Using both
        try:
            atms = read(input_string)
            return True, atms.get_positions(), atms.get_chemical_symbols()
        except FileNotFoundError:
            proceed, pos, sym = read_smiles(input_string)
            if not proceed:
                print("ERROR: File '%s' not found and not a valid SMILES code!" % input_string)
                return False, None, None
            return proceed, pos, sym


def read_smiles(smiles):
    mol = MolFromSmiles(smiles)
    if mol is None:
        return False, None, None
    mol = AddHs(mol)
    AllChem.EmbedMolecule(mol)
    try:
        return read_xyz_block(MolToXYZBlock(mol).strip().split("\n"))
    except ValueError:
        return False, None, None


def read_xyz(handle):
    content = handle.readlines()
    try:
        return read_xyz_block(content)
    except ValueError:
        return False, None, None


def read_xyz_block(string_list):
    try:
        atms = int(string_list[0])
    except ValueError:
        print("XYZ FORMAT ERROR: Could not read atom number.")
        raise ValueError
    pos, sym = [], []
    for line in string_list[2:atms + 2]:
        work = line.strip().split()
        try:
            sym.append(work[0])
            pos.append([float(work[1]), float(work[2]), float(work[3])])
        except IndexError:
            print("XYZ FORMAT ERROR: Line '%s' is not formatted correctly." % line)
            raise ValueError
    return True, pos, sym



import argparse

from asciimol.app import map_colors, map_radii
from asciimol.app.colors import init_curses_color_pairs
from asciimol.app.io import handle_io


class Config:
    """
    The runtime configuration object. Contains currently opened file, as well as any other configurables.
    """

    def __init__(self):
        self.coordinates = None
        self.symbols = None
        self.colors = None
        self.bonds = None

    def parse(self):
        parser = argparse.ArgumentParser()
        parser.add_argument('XYZFILE', metavar='XYZFILE or SMILES', type=str,
                            help='Specify an .xyz file or a SMILES string (e.g., CC) to open and display.')
        opts = parser.parse_args()
        proceed, self.coordinates, self.symbols = handle_io(opts.XYZFILE)
        return proceed

    def post_setup(self):
        self._setup_bonds()
        self._setup_colors()

    def _setup_bonds(self):
        if self.bonds:
            return
        atms = len(self.symbols)
        radii = list(map_radii(self.symbols))
        bonds = []
        unbound = list(range(atms))
        for i in range(atms):
            for j in range(i):
                xa, ya, za = self.coordinates[i]
                xb, yb, zb = self.coordinates[j]
                rsq = (radii[i] + radii[j] + 0.41) ** 2
                dist = (xa - xb) ** 2 + (ya - yb) ** 2 + (za - zb) ** 2
                if dist < rsq or dist < 0.4:
                    bonds.append((i, j))
                    unbound[i] = -1
                    unbound[j] = -1
        for n, state in enumerate(unbound):
            if state != -1:
                bonds += [(n, n)]
        self.bonds = bonds

    def _setup_colors(self):
        if self.colors:
            return
        colors = list(map_colors(self.symbols))
        self.colors = list(init_curses_color_pairs(colors))
